@@ -11,6 +11,7 @@ from os.path import join
 from . import core
 from .compat import int_types
 from .compat import ustr
+from .compat import WIN32
 from .decorators import memoize
 from .interaction import Interaction
 
@@ -67,6 +68,10 @@ def read_git_file(path):
         data = core.read(path).strip()
         if data.startswith(header):
             result = data[len(header):]
+            if result and not os.path.isabs(result):
+                path_folder = os.path.dirname(path)
+                repo_relative = os.path.join(path_folder, result)
+                result = os.path.normpath(repo_relative)
     return result
 
 
@@ -229,13 +234,15 @@ class Git(object):
         # Guard against thread-unsafe .git/index.lock files
         if not _readonly:
             INDEX_LOCK.acquire()
-        status, out, err = core.run_command(
-                command, cwd=_cwd, encoding=_encoding,
-                stdin=_stdin, stdout=_stdout, stderr=_stderr,
-                no_win32_startupinfo=_no_win32_startupinfo, **extra)
-        # Let the next thread in
-        if not _readonly:
-            INDEX_LOCK.release()
+        try:
+            status, out, err = core.run_command(
+                    command, cwd=_cwd, encoding=_encoding,
+                    stdin=_stdin, stdout=_stdout, stderr=_stderr,
+                    no_win32_startupinfo=_no_win32_startupinfo, **extra)
+        finally:
+            # Let the next thread in
+            if not _readonly:
+                INDEX_LOCK.release()
 
         if not _raw and out is not None:
             out = out.rstrip('\n')
@@ -325,9 +332,14 @@ class Git(object):
                 # case of argv overflow. we should be safe from that but use
                 # defensive coding for the worst-case scenario. on other OS-en
                 # we have ENAMETOOLONG which doesn't exist in with32 API.
-                status, out, err = self.execute(['git', '--version'])
-                if status == 0:
-                    raise e
+                try:
+                    status, out, err = self.execute(['git', '--version'])
+                except OSError:
+                    pass # git command is not available
+                else:
+                    if status == 0:
+                        # git is ok, there is something else...
+                        raise e
 
             core.stderr("error: unable to execute 'git'\n"
                         "error: please ensure that 'git' is in your $PATH")
