@@ -13,7 +13,9 @@ from .git import STDOUT
 from .i18n import N_
 
 
+# Object ID / SHA1-related constants
 EMPTY_TREE_OID = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'
+OID_LENGTH = 40
 
 
 class InvalidRepositoryError(Exception):
@@ -36,10 +38,43 @@ def diff_index_filenames(ref):
 def diff_filenames(*args):
     """Return a list of filenames that have been modified"""
     out = git.diff_tree(name_only=True, no_commit_id=True, r=True, z=True,
-                        *args)[STDOUT]
+                        _readonly=True, *args)[STDOUT]
     return _parse_diff_filenames(out)
 
 
+def listdir(dirname, ref='HEAD'):
+    """Get the contents of a directory according to Git
+
+    Query Git for the content of a directory, taking ignored
+    files into account.
+
+    """
+    dirs = []
+    files = []
+
+    # first, parse git ls-tree to get the tracked files
+    # in a list of (type, path) tuples
+    entries = ls_tree(dirname, ref=ref)
+    for entry in entries:
+        if entry[0][0] == 't':  # tree
+            dirs.append(entry[1])
+        else:
+            files.append(entry[1])
+
+    # gather untracked files
+    untracked = untracked_files(paths=[dirname], directory=True)
+    for path in untracked:
+        if path.endswith('/'):
+            dirs.append(path[:-1])
+        else:
+            files.append(path)
+
+    dirs.sort()
+    files.sort()
+
+    return (dirs, files)
+
+
 def diff(args):
     """Return a list of filenames for the given diff arguments
 
@@ -146,9 +181,19 @@ def branch_list(remote=False):
         return for_each_ref_basename('refs/heads')
 
 
+def _version_sort():
+    if version.check_git('version-sort'):
+        sort = 'version:refname'
+    else:
+        sort = False
+    return sort
+
+
 def for_each_ref_basename(refs, git=git):
     """Return refs starting with 'refs'."""
-    out = git.for_each_ref(refs, format='%(refname)')[STDOUT]
+    sort = _version_sort()
+    status, out, err = git.for_each_ref(refs, format='%(refname)',
+                                        sort=sort, _readonly=True)
     output = out.splitlines()
     non_heads = [x for x in output if not x.endswith('/HEAD')]
     return list(map(lambda x: x[len(refs) + 1:], non_heads))
@@ -167,12 +212,15 @@ def all_refs(split=False, git=git):
     query = (triple('refs/tags', tags),
              triple('refs/heads', local_branches),
              triple('refs/remotes', remote_branches))
-    out = git.for_each_ref(format='%(refname)')[STDOUT]
+    sort = _version_sort()
+    status, out, err = git.for_each_ref(format='%(refname)',
+                                        sort=sort, _readonly=True)
     for ref in out.splitlines():
         for prefix, prefix_len, dst in query:
             if ref.startswith(prefix) and not ref.endswith('/HEAD'):
                 dst.append(ref[prefix_len:])
                 continue
+    tags.reverse()
     if split:
         return local_branches, remote_branches, tags
     else:
@@ -199,14 +247,14 @@ def tracked_branch(branch=None, config=None):
     return None
 
 
-def untracked_files(git=git, paths=None):
+def untracked_files(git=git, paths=None, **kwargs):
     """Returns a sorted list of untracked files."""
 
     if paths is None:
         paths = []
     args = ['--'] + paths
     out = git.ls_files(z=True, others=True, exclude_standard=True,
-                       *args)[STDOUT]
+                       *args, **kwargs)[STDOUT]
     if out:
         return out[:-1].split('\0')
     return []
@@ -214,12 +262,14 @@ def untracked_files(git=git, paths=None):
 
 def tag_list():
     """Return a list of tags."""
-    return list(reversed(for_each_ref_basename('refs/tags')))
+    result = for_each_ref_basename('refs/tags')
+    result.reverse()
+    return result
 
 
 def log(git, *args, **kwargs):
     return git.log(no_color=True, no_abbrev_commit=True,
-                   no_ext_diff=True, *args, **kwargs)[STDOUT]
+                   no_ext_diff=True, _readonly=True, *args, **kwargs)[STDOUT]
 
 
 def commit_diff(oid, git=git):
@@ -272,7 +322,8 @@ def oid_diff(git, oid, filename=None):
         # "git show" is clever enough to handle the root commit.
         args = [oid + '^!']
         _add_filename(args, filename)
-        status, out, err = git.show(pretty='format:', *args, **opts)
+        status, out, err = git.show(pretty='format:', _readonly=True,
+                                    *args, **opts)
         out = out.lstrip()
     return out
 
@@ -329,13 +380,14 @@ def diff_helper(commit=None,
         else:
             return ''
 
-    return extract_diff_header(status, deleted,
-                               with_diff_header, suppress_header, out)
+    result = extract_diff_header(status, deleted,
+                                 with_diff_header, suppress_header, out)
+    return core.UStr(result, out.encoding)
 
 
 def extract_diff_header(status, deleted,
                         with_diff_header, suppress_header, diffoutput):
-    headers = []
+    """Split a diff into a header section and payload section"""
 
     if diffoutput.startswith('Submodule'):
         if with_diff_header:
@@ -345,27 +397,31 @@ def extract_diff_header(status, deleted,
 
     start = False
     del_tag = 'deleted file mode '
-    output = StringIO()
 
-    diff = diffoutput.split('\n')
-    for line in diff:
+    output = StringIO()
+    headers = StringIO()
+
+    for line in diffoutput.split('\n'):
         if not start and '@@' == line[:2] and '@@' in line[2:]:
             start = True
         if start or (deleted and del_tag in line):
             output.write(line + '\n')
         else:
             if with_diff_header:
-                headers.append(line)
+                headers.write(line + '\n')
             elif not suppress_header:
                 output.write(line + '\n')
 
-    result = output.getvalue()
+    output_text = output.getvalue()
     output.close()
 
+    headers_text = headers.getvalue()
+    headers.close()
+
     if with_diff_header:
-        return('\n'.join(headers), result)
+        return(headers_text, output_text)
     else:
-        return result
+        return output_text
 
 
 def format_patchsets(to_export, revs, output='patches'):
@@ -410,7 +466,7 @@ def format_patchsets(to_export, revs, output='patches'):
     for patchset in patches_to_export:
         stat, out, err = export_patchset(patchset[0],
                                          patchset[-1],
-                                         output='patches',
+                                         output=output,
                                          n=len(patchset) > 1,
                                          thread=True,
                                          patch_with_stat=True)
@@ -563,7 +619,7 @@ def _branch_status(branch):
 
 def merge_base(head, ref):
     """Given `ref`, return $(git merge-base ref HEAD)..ref."""
-    return git.merge_base(head, ref)[STDOUT]
+    return git.merge_base(head, ref, _readonly=True)[STDOUT]
 
 
 def merge_base_parent(branch):
@@ -576,7 +632,7 @@ def merge_base_parent(branch):
 def parse_ls_tree(rev):
     """Return a list of (mode, type, oid, path) tuples."""
     output = []
-    lines = git.ls_tree(rev, r=True)[STDOUT].splitlines()
+    lines = git.ls_tree(rev, r=True, _readonly=True)[STDOUT].splitlines()
     regex = re.compile(r'^(\d+)\W(\w+)\W(\w+)[ \t]+(.*)$')
     for line in lines:
         match = regex.match(line)
@@ -589,6 +645,24 @@ def parse_ls_tree(rev):
     return output
 
 
+def ls_tree(path, ref='HEAD'):
+    """Return a parsed git ls-tree result for a single directory"""
+
+    result = []
+    status, out, err = git.ls_tree(ref, '--', path, z=True, full_tree=True)
+    if status == 0 and out:
+        for line in out[:-1].split('\0'):
+            # .....6 ...4 ......................................40
+            # 040000 tree c127cde9a0c644a3a8fef449a244f47d5272dfa6	relative
+            # 100644 blob 139e42bf4acaa4927ec9be1ec55a252b97d3f1e2	relative/path
+            # 0..... 7... 12......................................	53
+            # path offset = 6 + 1 + 4 + 1 + 40 + 1 = 53
+            objtype = line[7:11]
+            relpath = line[53:]
+            result.append((objtype, relpath))
+
+    return result
+
 # A regex for matching the output of git(log|rev-list) --pretty=oneline
 REV_LIST_REGEX = re.compile(r'^([0-9a-f]{40}) (.*)$')
 
@@ -655,7 +729,7 @@ def prepare_commit_message_hook(config=None):
 def abort_merge():
     """Abort a merge by reading the tree at HEAD."""
     # Reset the worktree
-    git.read_tree('HEAD', reset=True, u=True, v=True)
+    status, out, err = git.read_tree('HEAD', reset=True, u=True, v=True)
     # remove MERGE_HEAD
     merge_head = git.git_path('MERGE_HEAD')
     if core.exists(merge_head):
@@ -665,6 +739,7 @@ def abort_merge():
     while merge_msg_path:
         core.unlink(merge_msg_path)
         merge_msg_path = merge_message_path()
+    return status, out, err
 
 
 def strip_remote(remotes, remote_branch):
@@ -673,3 +748,13 @@ def strip_remote(remotes, remote_branch):
         if remote_branch.startswith(prefix):
             return remote_branch[len(prefix):]
     return remote_branch.split('/', 1)[-1]
+
+
+def parse_refs(argv):
+    """Parse command-line arguments into object IDs"""
+    status, out, err = git.rev_parse(*argv)
+    if status == 0:
+        oids = [oid for oid in out.splitlines() if oid]
+    else:
+        oids = argv
+    return oids
