@@ -1,7 +1,5 @@
 """Launcher and command line interface to git-cola"""
-
 from __future__ import absolute_import, division, unicode_literals
-
 import argparse
 import os
 import sys
@@ -9,7 +7,6 @@ import sys
 from .app import add_common_arguments
 from .app import application_init
 from .app import application_start
-
 from . import cmds
 from . import compat
 from . import core
@@ -54,6 +51,7 @@ def parse_args(argv):
     add_pull_command(subparser)
     add_push_command(subparser)
     add_rebase_command(subparser)
+    add_recent_command(subparser)
     add_remote_command(subparser)
     add_search_command(subparser)
     add_stash_command(subparser)
@@ -81,7 +79,7 @@ def add_cola_command(subparser):
 
 
 def add_about_command(parent):
-    parser = add_command(parent, 'about', 'about git-cola', cmd_about)
+    add_command(parent, 'about', 'about git-cola', cmd_about)
 
 def add_am_command(parent):
     parser = add_command(parent, 'am', 'apply patches using "git am"', cmd_am)
@@ -140,7 +138,9 @@ def add_grep_command(subparser):
 
 
 def add_merge_command(subparser):
-    add_command(subparser, 'merge', 'merge branches', cmd_merge)
+    parser = add_command(subparser, 'merge', 'merge branches', cmd_merge)
+    parser.add_argument('ref', nargs='?', metavar='<ref>',
+                        help='branch, tag, or commit to merge')
 
 
 def add_pull_command(subparser):
@@ -243,6 +243,9 @@ def add_rebase_command(subparser):
                              '"git checkout <branch>" before doing anything '
                              'else when <branch> is specified')
 
+def add_recent_command(subparser):
+    add_command(subparser, 'recent', 'edit recent files', cmd_recent)
+
 
 def add_remote_command(subparser):
     add_command(subparser, 'remote', 'edit remotes', cmd_remote)
@@ -305,11 +308,11 @@ def cmd_am(args):
 
 
 def cmd_archive(args):
-    from .widgets.archive import GitArchiveDialog
+    from .widgets.archive import Archive
     context = application_init(args, update=True)
     if args.ref is None:
         args.ref = context.model.currentbranch
-    view = GitArchiveDialog(args.ref)
+    view = Archive(args.ref)
     return application_start(context, view)
 
 
@@ -377,8 +380,8 @@ def cmd_grep(args):
 
 def cmd_merge(args):
     context = application_init(args, update=True)
-    from .widgets.merge import MergeView
-    view = MergeView(context.cfg, context.model, parent=None)
+    from .widgets.merge import Merge
+    view = Merge(context.cfg, context.model, parent=None, ref=args.ref)
     return application_start(context, view)
 
 
@@ -436,16 +439,18 @@ def cmd_rebase(args):
             'edit_todo': args.edit_todo,
             'upstream': args.upstream,
             'branch': args.branch,
-            'capture_output': False,
     }
     status, out, err = cmds.do(cmds.Rebase, **kwargs)
-    if out:
-        core.stdout(out)
-    if err:
-        core.stderr(err)
     return status
 
 
+def cmd_recent(args):
+    from .widgets import recent
+    context = application_init(args)
+    view = recent.browse_recent_files()
+    return application_start(context, view)
+
+
 def cmd_remote(args):
     from .widgets import editremotes
     context = application_init(args)
