@@ -66,6 +66,7 @@ class MainModel(Observable):
         # Initialize the git command object
         self.git = git.current()
 
+        self.initialized = False
         self.head = 'HEAD'
         self.diff_text = ''
         self.mode = self.mode_none
@@ -191,6 +192,7 @@ class MainModel(Observable):
     def update_status(self, update_index=False):
         # Give observers a chance to respond
         self.notify_observers(self.message_about_to_update)
+        self.initialized = True
         self._update_merge_rebase_status()
         self._update_files(update_index=update_index)
         self._update_remotes()
@@ -243,8 +245,10 @@ class MainModel(Observable):
         self.tags = tags
 
     def _update_merge_rebase_status(self):
-        self.is_merging = core.exists(self.git.git_path('MERGE_HEAD'))
-        self.is_rebasing = core.exists(self.git.git_path('rebase-merge'))
+        merge_head = self.git.git_path('MERGE_HEAD')
+        rebase_merge = self.git.git_path('rebase-merge')
+        self.is_merging = merge_head and core.exists(merge_head)
+        self.is_rebasing = rebase_merge and core.exists(rebase_merge)
         if self.is_merging and self.mode == self.mode_amend:
             self.set_mode(self.mode_none)
 
@@ -464,7 +468,9 @@ class MainModel(Observable):
         remove = []
 
         for path in set(paths):
-            if core.exists(path):
+            if core.exists(path) or core.islink(path):
+                if path.endswith('/'):
+                    path = path.rstrip('/')
                 add.append(path)
             else:
                 remove.append(path)
@@ -515,7 +521,8 @@ def remote_args(remote,
                 rebase=False,
                 pull=False,
                 push=False,
-                set_upstream=False):
+                set_upstream=False,
+                prune=False):
     """Return arguments for git fetch/push/pull"""
 
     args = [remote]
@@ -540,6 +547,8 @@ def remote_args(remote,
         kwargs['set_upstream'] = True
     if tags:
         kwargs['tags'] = True
+    if prune:
+        kwargs['prune'] = True
 
     return (args, kwargs)
 
