@@ -24,12 +24,14 @@ MERGE_KEEPBACKUP = 'merge.keepbackup'
 MERGE_SUMMARY = 'merge.summary'
 MERGE_VERBOSITY = 'merge.verbosity'
 MERGETOOL = 'merge.tool'
+EXPANDTAB = 'cola.expandtab'
 SAVEWINDOWSETTINGS = 'cola.savewindowsettings'
 SORT_BOOKMARKS = 'cola.sortbookmarks'
 TABWIDTH = 'cola.tabwidth'
 TEXTWIDTH = 'cola.textwidth'
 USER_EMAIL = 'user.email'
 USER_NAME = 'user.name'
+SPELL_CHECK = 'cola.spellcheck'
 
 
 def default_blame_viewer():
@@ -87,6 +89,14 @@ def linebreak():
     return gitcfg.current().get(LINEBREAK, True)
 
 
+def spellcheck():
+    return gitcfg.current().get(SPELL_CHECK, False)
+
+
+def expandtab():
+    return gitcfg.current().get(EXPANDTAB, False)
+
+
 def sort_bookmarks():
     return gitcfg.current().get(SORT_BOOKMARKS, True)
 
