@@ -13,6 +13,10 @@ from .git import git
 from .git import STDOUT
 from .decorators import memoize
 from ._version import VERSION
+try:
+    from ._build_version import BUILD_VERSION
+except:
+    BUILD_VERSION = ''
 
 # minimum version requirements
 _versions = {
@@ -26,6 +30,8 @@ _versions = {
     # git check-ignore was introduced in 1.8.2, but did not follow the same
     # rules as git add and git status until 1.8.5
     'check-ignore': '1.8.5',
+    # git for-each-ref --sort=version:refname
+    'version-sort': '2.7.0',
 }
 
 
@@ -39,6 +45,11 @@ def version():
     return VERSION
 
 
+def build_version():
+    """Return the build version, which includes the Git ID"""
+    return BUILD_VERSION
+
+
 @memoize
 def check_version(min_ver, ver):
     """Check whether ver is greater or equal to min_ver
@@ -54,6 +65,11 @@ def check(key, ver):
     return check_version(get(key), ver)
 
 
+def check_git(key):
+    """Checks if Git has a specific feature"""
+    return check(key, git_version())
+
+
 def version_to_list(version):
     """Convert a version string to a list of numbers or strings
     """
