@@ -39,3 +39,6 @@ logo_text = 24
 radio_border = max(1, scale(1.0 - (1.0 / math.pi)))
 
 separator = scale(3)
+
+dialog_w = scale(720)
+dialog_h = scale(445)
