@@ -11,6 +11,7 @@ from .. import gitcmds
 from .. import icons
 from .. import qtutils
 from . import completion
+from . import standard
 from . import defs
 
 
@@ -18,31 +19,17 @@ def local_merge():
     """Provides a dialog for merging branches"""
     model = main.model()
     cfg = gitcfg.current()
-    view = MergeView(cfg, model, qtutils.active_window())
+    view = Merge(cfg, model, qtutils.active_window())
     view.show()
     view.raise_()
     return view
 
 
-def abort_merge():
-    """Prompts before aborting a merge in progress
-    """
-    title = N_('Abort Merge...')
-    txt = N_('Aborting the current merge will cause '
-             '*ALL* uncommitted changes to be lost.\n'
-             'Recovering uncommitted changes is not possible.')
-    info_txt = N_('Aborting the current merge?')
-    ok_txt = N_('Abort Merge')
-    if qtutils.confirm(title, txt, info_txt, ok_txt,
-                       default=False, icon=icons.undo()):
-        gitcmds.abort_merge()
-
-
-class MergeView(QtWidgets.QDialog):
+class Merge(standard.Dialog):
     """Provides a dialog for merging branches."""
 
-    def __init__(self, cfg, model, parent=None):
-        QtWidgets.QDialog.__init__(self, parent)
+    def __init__(self, cfg, model, parent=None, ref=None):
+        standard.Dialog.__init__(self, parent=parent)
         self.cfg = cfg
         self.model = model
         if parent is not None:
@@ -54,12 +41,12 @@ class MergeView(QtWidgets.QDialog):
         self.revision_label.setText(N_('Revision to Merge'))
 
         self.revision = completion.GitRefLineEdit()
-        self.revision.setFocus()
         self.revision.setToolTip(N_('Revision to Merge'))
+        if ref:
+            self.revision.set_value(ref)
 
         self.radio_local = qtutils.radio(text=N_('Local Branch'), checked=True)
         self.radio_remote = qtutils.radio(text=N_('Tracking Branch'))
-
         self.radio_tag = qtutils.radio(text=N_('Tag'))
 
         self.revisions = QtWidgets.QListWidget()
@@ -92,7 +79,8 @@ class MergeView(QtWidgets.QDialog):
         self.button_close = qtutils.close_button()
 
         icon = icons.merge()
-        self.button_merge = qtutils.create_button(text=N_('Merge'), icon=icon)
+        self.button_merge = qtutils.create_button(text=N_('Merge'), icon=icon,
+                                                  default=True)
 
         # Layouts
         self.revlayt = qtutils.hbox(defs.no_margin, defs.spacing,
@@ -104,10 +92,10 @@ class MergeView(QtWidgets.QDialog):
                                       self.radio_tag)
 
         self.buttonlayt = qtutils.hbox(defs.no_margin, defs.button_spacing,
-                                       self.button_viz, qtutils.STRETCH,
+                                       self.button_close, qtutils.STRETCH,
                                        self.checkbox_squash, self.checkbox_noff,
                                        self.checkbox_commit, self.checkbox_sign,
-                                       self.button_close, self.button_merge)
+                                       self.button_viz, self.button_merge)
 
         self.mainlayt = qtutils.vbox(defs.margin, defs.spacing,
                                      self.radiolayt, self.revisions,
@@ -116,20 +104,23 @@ class MergeView(QtWidgets.QDialog):
 
         # Signal/slot connections
         self.revision.textChanged.connect(self.update_title)
+        self.revision.returnPressed.connect(self.merge_revision)
         self.revisions.itemSelectionChanged.connect(self.revision_selected)
 
-        qtutils.connect_button(self.button_close, self.reject)
-        qtutils.connect_button(self.checkbox_squash, self.toggle_squash)
-        qtutils.connect_button(self.radio_local, self.update_revisions)
-        qtutils.connect_button(self.radio_remote, self.update_revisions)
-        qtutils.connect_button(self.radio_tag, self.update_revisions)
+        qtutils.connect_released(self.radio_local, self.update_revisions)
+        qtutils.connect_released(self.radio_remote, self.update_revisions)
+        qtutils.connect_released(self.radio_tag, self.update_revisions)
         qtutils.connect_button(self.button_merge, self.merge_revision)
+        qtutils.connect_button(self.checkbox_squash, self.toggle_squash)
         qtutils.connect_button(self.button_viz, self.viz_revision)
+        qtutils.connect_button(self.button_close, self.reject)
 
         # Observer messages
         model.add_observer(model.message_updated, self.update_all)
         self.update_all()
-        self.resize(700, 400)
+
+        self.init_size(parent=parent)
+        self.revision.setFocus(True)
 
     def update_all(self):
         """Set the branch name for the window title and label."""
@@ -144,6 +135,7 @@ class MergeView(QtWidgets.QDialog):
                    dict(revision=revision, branch=branch))
         else:
             txt = N_('Merge into "%s"') % branch
+        self.button_merge.setEnabled(bool(revision))
         self.title_label.setText(txt)
         self.setWindowTitle(txt)
 
@@ -210,3 +202,19 @@ class MergeView(QtWidgets.QDialog):
         sign = self.checkbox_sign.isChecked()
         cmds.do(cmds.Merge, revision, no_commit, squash, noff, sign)
         self.accept()
+
+    def export_state(self):
+        """Export persistent settings"""
+        state = super(Merge, self).export_state()
+        state['no-ff'] = self.checkbox_noff.isChecked()
+        state['sign'] = self.checkbox_sign.isChecked()
+        state['commit'] = self.checkbox_commit.isChecked()
+        return state
+
+    def apply_state(self, state):
+        """Apply persistent settings"""
+        result = super(Merge, self).apply_state(state)
+        self.checkbox_noff.setChecked(state.get('no-ff', False))
+        self.checkbox_sign.setChecked(state.get('sign', False))
+        self.checkbox_commit.setChecked(state.get('commit', True))
+        return result
