@@ -1,5 +1,7 @@
 #! /usr/bin/env python
- 
+
+from __future__ import absolute_import
+
 from optparse import OptionParser
 
 import os
@@ -8,6 +10,10 @@ import shutil
 import codecs
 
 
+def stdout(msg):
+    sys.stdout.write(msg + '\n')
+
+
 class DirHelper(object):
 
     def __init__(self, is_dir, list_dir, walk, rmtree):
@@ -17,6 +23,7 @@ class DirHelper(object):
         self.walk = walk
         self.rmtree = rmtree
 
+
 class FileSystemHelper(object):
 
     def __init__(self, open_, path_join, move, exists):
@@ -26,6 +33,7 @@ class FileSystemHelper(object):
         self.move = move
         self.exists = exists
 
+
 class Replacer(object):
     "Encapsulates a simple text replace"
 
@@ -38,6 +46,7 @@ class Replacer(object):
 
         return text.replace( self.from_, self.to )
 
+
 class FileHandler(object):
     "Applies a series of replacements the contents of a file inplace"
 
@@ -56,6 +65,7 @@ class FileHandler(object):
 
         self.opener(self.name, "w").write(text)
 
+
 class Remover(object):
 
     def __init__(self, exists, remove):
@@ -276,12 +286,12 @@ def sphinx_extension(app, exception):
 
     if not app.config.sphinx_to_github:
         if app.config.sphinx_to_github_verbose:
-            print("Sphinx-to-github: Disabled, doing nothing.")
+            stdout("Sphinx-to-github: Disabled, doing nothing.")
         return
 
     if exception:
         if app.config.sphinx_to_github_verbose:
-            print("Sphinx-to-github: Exception raised in main build, doing nothing.")
+            stdout("Sphinx-to-github: Exception raised in main build, doing nothing.")
         return
 
     dir_helper = DirHelper(
@@ -357,7 +367,7 @@ def main(args):
             shutil.move,
             os.path.exists
             )
-    
+
     operations_factory = OperationsFactory()
     handler_factory = HandlerFactory()
 
@@ -373,11 +383,7 @@ def main(args):
 
     layout = layout_factory.create_layout(path)
     layout.process()
-    
 
 
 if __name__ == "__main__":
     main(sys.argv[1:])
-
-
-
