@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# -*- encoding: utf-8 -*-
+# encoding: utf-8
 
-from __future__ import unicode_literals
+from __future__ import absolute_import, division, unicode_literals
 
 import os
 import unittest
