@@ -4,11 +4,13 @@
 from __future__ import absolute_import, division, unicode_literals
 
 import os
-import signal
 import time
 import unittest
 
-from mock import patch
+try:
+    from unittest.mock import patch
+except ImportError:
+    from mock import patch
 
 from cola import git
 from cola.compat import WIN32
@@ -226,8 +228,8 @@ class GitCommandTest(unittest.TestCase):
 
     def test_show(self):
         """Test running 'git show'"""
-        sha = 'HEAD'
-        content = self.git.show(sha)[STDOUT]
+        oid = 'HEAD'
+        content = self.git.show(oid)[STDOUT]
         self.failUnless(content.startswith('commit '))
 
     def test_stdout(self):
@@ -276,20 +278,6 @@ class GitCommandTest(unittest.TestCase):
         self.assertEqual(out, '\0' * (1024 * 16 + 1))
         self.assertEqual(err, '\0' * (1024 * 16 + 1))
 
-    def test_it_handles_interrupted_syscalls(self):
-        """Test that we handle interrupted system calls"""
-        # send ourselves a signal that causes EINTR
-        if WIN32:
-            # SIGALRM not supported on Windows
-            return
-        prev_handler = signal.signal(signal.SIGALRM, lambda x, y: 1)
-        signal.alarm(1)
-        time.sleep(0.1)
-        status, out, err = git.Git.execute(['sleep', '1'])
-        self.assertEqual(status, 0)
-
-        signal.signal(signal.SIGALRM, prev_handler)
-
 
 if __name__ == '__main__':
     unittest.main()
